mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: Len Brown <lenb@kernel.org>
To: linux-pm@lists.linux-foundation.org
Cc: linux-acpi@vger.kernel.org, linux-kernel@vger.kernel.org,
	"Rafael J. Wysocki" <rjw@sisk.pl>,
	Len Brown <len.brown@intel.com>
Subject: [PATCH 30/37] Hibernation: Introduce begin() and end() callbacks
Date: Thu, 31 Jan 2008 23:37:01 -0500	[thread overview]
Message-ID: <07f44315b9cfad180296df99bcd38879f063f092.1201840183.git.len.brown@intel.com> (raw)
In-Reply-To: <1201840628-23136-1-git-send-email-lenb@kernel.org>
In-Reply-To: <edae89f5d1eb1fbe1cab74333db5a8e29776900e.1201840183.git.len.brown@intel.com>

From: Rafael J. Wysocki <rjw@sisk.pl>

Introduce global hibernation callback .end() and rename global
hibernation callback .start() to .begin(), in analogy with the
recent modifications of the global suspend callbacks.

Signed-off-by: Rafael J. Wysocki <rjw@sisk.pl>
Signed-off-by: Len Brown <len.brown@intel.com>
---
 drivers/acpi/sleep/main.c |   14 ++++++++++++--
 include/linux/suspend.h   |   14 +++++++++-----
 kernel/power/disk.c       |   33 ++++++++++++++++++++++++---------
 3 files changed, 45 insertions(+), 16 deletions(-)

diff --git a/drivers/acpi/sleep/main.c b/drivers/acpi/sleep/main.c
index c37c4ea..31e8e58 100644
--- a/drivers/acpi/sleep/main.c
+++ b/drivers/acpi/sleep/main.c
@@ -281,7 +281,7 @@ static struct dmi_system_id __initdata acpisleep_dmi_table[] = {
 #endif /* CONFIG_SUSPEND */
 
 #ifdef CONFIG_HIBERNATION
-static int acpi_hibernation_start(void)
+static int acpi_hibernation_begin(void)
 {
 	acpi_target_sleep_state = ACPI_STATE_S4;
 	return 0;
@@ -341,6 +341,15 @@ static void acpi_hibernation_finish(void)
 	acpi_target_sleep_state = ACPI_STATE_S0;
 }
 
+static void acpi_hibernation_end(void)
+{
+	/*
+	 * This is necessary in case acpi_hibernation_finish() is not called
+	 * during a failing transition to the sleep state.
+	 */
+	acpi_target_sleep_state = ACPI_STATE_S0;
+}
+
 static int acpi_hibernation_pre_restore(void)
 {
 	acpi_status status;
@@ -356,7 +365,8 @@ static void acpi_hibernation_restore_cleanup(void)
 }
 
 static struct platform_hibernation_ops acpi_hibernation_ops = {
-	.start = acpi_hibernation_start,
+	.begin = acpi_hibernation_begin,
+	.end = acpi_hibernation_end,
 	.pre_snapshot = acpi_hibernation_prepare,
 	.finish = acpi_hibernation_finish,
 	.prepare = acpi_hibernation_prepare,
diff --git a/include/linux/suspend.h b/include/linux/suspend.h
index a0b1dbb..646ce2d 100644
--- a/include/linux/suspend.h
+++ b/include/linux/suspend.h
@@ -136,14 +136,17 @@ extern void mark_free_pages(struct zone *zone);
 /**
  * struct platform_hibernation_ops - hibernation platform support
  *
- * The methods in this structure allow a platform to override the default
- * mechanism of shutting down the machine during a hibernation transition.
+ * The methods in this structure allow a platform to carry out special
+ * operations required by it during a hibernation transition.
  *
- * All three methods must be assigned.
+ * All the methods below must be implemented.
  *
  * @start: Tell the platform driver that we're starting hibernation.
  *	Called right after shrinking memory and before freezing devices.
  *
+ * @end: Called by the PM core right after resuming devices, to indicate to
+ *	the platform that the system has returned to the working state.
+ *
  * @pre_snapshot: Prepare the platform for creating the hibernation image.
  *	Called right after devices have been frozen and before the nonboot
  *	CPUs are disabled (runs with IRQs on).
@@ -178,7 +181,8 @@ extern void mark_free_pages(struct zone *zone);
  *	thawing devices (runs with IRQs on).
  */
 struct platform_hibernation_ops {
-	int (*start)(void);
+	int (*begin)(void);
+	void (*end)(void);
 	int (*pre_snapshot)(void);
 	void (*finish)(void);
 	int (*prepare)(void);
@@ -136,14 +136,17 @@ extern void mark_free_pages(struct zone *zone);
  *
- * @start: Tell the platform driver that we're starting hibernation.
+ * @begin: Tell the platform driver that we're starting hibernation.
  *	Called right after shrinking memory and before freezing devices.
  *
diff --git a/kernel/power/disk.c b/kernel/power/disk.c
index 64e42ab..53c22d9 100644
--- a/kernel/power/disk.c
+++ b/kernel/power/disk.c
@@ -54,8 +54,8 @@ static struct platform_hibernation_ops *hibernation_ops;
 
 void hibernation_set_ops(struct platform_hibernation_ops *ops)
 {
-	if (ops && !(ops->start && ops->pre_snapshot && ops->finish
-	    && ops->prepare && ops->enter && ops->pre_restore
+	if (ops && !(ops->begin && ops->end &&  ops->pre_snapshot
+	    && ops->prepare && ops->finish && ops->enter && ops->pre_restore
 	    && ops->restore_cleanup)) {
 		WARN_ON(1);
 		return;
@@ -100,14 +100,25 @@ static int hibernation_test(int level) { return 0; }
 #endif /* !CONFIG_PM_DEBUG */
 
 /**
  *	platform_start - tell the platform driver that we're starting
  *	hibernation
  */
 
-static int platform_start(int platform_mode)
+static int platform_begin(int platform_mode)
 {
 	return (platform_mode && hibernation_ops) ?
-		hibernation_ops->start() : 0;
+		hibernation_ops->begin() : 0;
+}
+
+/**
+ *	platform_end - tell the platform driver that we've entered the
+ *	working state
+ */
+
+static void platform_end(int platform_mode)
+{
+	if (platform_mode && hibernation_ops)
+		hibernation_ops->end();
 }
 
 /**
@@ -237,9 +248,9 @@ int hibernation_snapshot(int platform_mode)
 	if (error)
 		return error;
 
-	error = platform_start(platform_mode);
+	error = platform_begin(platform_mode);
 	if (error)
-		return error;
+		goto Close;
 
 	suspend_console();
 	error = device_suspend(PMSG_FREEZE);
@@ -272,6 +283,8 @@ int hibernation_snapshot(int platform_mode)
 	device_resume();
  Resume_console:
 	resume_console();
+ Close:
+	platform_end(platform_mode);
 	return error;
 }
 
@@ -373,9 +386,9 @@ int hibernation_platform_enter(void)
 	 * hibernation_ops->finish() before saving the image, so we should let
 	 * the firmware know that we're going to enter the sleep state after all
 	 */
-	error = hibernation_ops->start();
+	error = hibernation_ops->begin();
 	if (error)
-		return error;
+		goto Close;
 
 	suspend_console();
 	error = device_suspend(PMSG_SUSPEND);
@@ -409,6 +422,8 @@ int hibernation_platform_enter(void)
 	device_resume();
  Resume_console:
 	resume_console();
+ Close:
+	hibernation_ops->end();
 	return error;
 }
 
@@ -100,14 +100,25 @@ static int hibernation_test(int level) { return 0; }
 
 /**
- *	platform_start - tell the platform driver that we're starting
+ *	platform_begin - tell the platform driver that we're starting
  *	hibernation
  */
-- 
1.5.4.rc5.16.gc0279


  parent reply	other threads:[~2008-02-01  4:52 UTC|newest]

Thread overview: 42+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2008-02-01  4:36 suspend patches for 2.6.25-rc0 Len Brown
2008-02-01  4:36 ` [PATCH 01/37] Hibernation: Introduce SNAPSHOT_GET_IMAGE_SIZE ioctl Len Brown
2008-02-01  4:36   ` [PATCH 02/37] Hibernation: Rework platform support ioctls (rev. 2) Len Brown
2008-02-01  4:36   ` [PATCH 03/37] Hibernation: Mark SNAPSHOT_SET_SWAP_FILE ioctl as deprecated " Len Brown
2008-02-01  4:36   ` [PATCH 04/37] Hibernation: Correct definitions of some ioctls " Len Brown
2008-02-01  4:36   ` [PATCH 05/37] Hibernation: Introduce exportable suspend ioctls header " Len Brown
2008-02-01  4:36   ` [PATCH 06/37] ACPI: Fix mismerge in acpi_hibernation_finish Len Brown
2008-02-01  4:36   ` [PATCH 07/37] Hibernation: Move function prototypes to header Len Brown
2008-02-01  4:36   ` [PATCH 08/37] Hibernation: Add PM_RESTORE_PREPARE and PM_POST_RESTORE notifiers (rev. 2) Len Brown
2008-02-01  4:36   ` [PATCH 09/37] Suspend: Testing facility " Len Brown
2008-02-01  4:36   ` [PATCH 10/37] suspend: build fix responding to 2.6.25 kset change Len Brown
2008-02-01  4:36   ` [PATCH 11/37] Hibernation: New testing facility (rev. 2) Len Brown
2008-02-01  4:36   ` [PATCH 12/37] PM: Suspend/hibernation debug documentation update " Len Brown
2008-02-01  4:36   ` [PATCH 13/37] PM: Make PM_TRACE more architecture independent Len Brown
2008-02-01  4:36   ` [PATCH 14/37] PM: Convert PM notifiers to out-of-line code Len Brown
2008-02-01  4:36   ` [PATCH 15/37] Suspend: Fix compilation warning for CONFIG_SUSPEND unset Len Brown
2008-02-01  4:36   ` [PATCH 16/37] Hibernation: Move low level resume to disk.c Len Brown
2008-02-01  4:36   ` [PATCH 17/37] Suspend: Fix comment in main.c Len Brown
2008-02-01  4:36   ` [PATCH 18/37] Hibernation: Fix comment in disk.c Len Brown
2008-02-01  4:36   ` [PATCH 19/37] Hibernation: Remove unnecessary variable declaration Len Brown
2008-02-01  4:36   ` [PATCH 20/37] Suspend: Use common prefix in messages Len Brown
2008-02-01  4:36   ` [PATCH 21/37] Hibernation: Update messages Len Brown
2008-02-01  4:36   ` [PATCH 22/37] Hibernation: Clean up Kconfig (V2) Len Brown
2008-02-01  4:36   ` [PATCH 23/37] Suspend: " Len Brown
2008-02-01  9:05     ` Bryan Wu
2008-02-04 14:59     ` Pavel Machek
2008-02-01  4:36   ` [PATCH 24/37] ACPI: clear GPE earily in resume to avoid warning Len Brown
2008-02-01  4:36   ` [PATCH 25/37] suspend: fix ia64 allmodconfig build Len Brown
2008-02-01  4:36   ` [PATCH 26/37] Suspend: Introduce begin() and end() callbacks Len Brown
2008-02-01  4:36   ` [PATCH 27/37] ACPI: Separate invocations of _GTS and _BFS from _PTS and _WAK Len Brown
2008-02-01  4:36   ` [PATCH 28/37] ACPI: Separate disabling of GPEs from _PTS Len Brown
2008-02-01  4:37   ` [PATCH 29/37] ACPI suspend: Call _PTS before suspending devices Len Brown
2008-02-01  4:37   ` Len Brown [this message]
2008-02-01  4:37   ` [PATCH 31/37] ACPI hibernation: " Len Brown
2008-02-01  4:37   ` [PATCH 32/37] ACPI: Print message before calling _PTS Len Brown
2008-02-01  4:37   ` [PATCH 33/37] Suspend: Add config option to disable the freezer if architecture wants that Len Brown
2008-02-01  4:37   ` [PATCH 34/37] Suspend: Clean up suspend_64.c Len Brown
2008-02-01  4:37   ` [PATCH 35/37] Suspend: Invoke suspend notifications after console switch Len Brown
2008-02-01  4:37   ` [PATCH 36/37] Hibernation: " Len Brown
2008-02-01  4:37   ` [PATCH 37/37] PM: Remove obsolete /sys/devices/.../power/state docs Len Brown
2008-02-01  5:48 ` [PATCH] feature-removal: Document suspend/hibernate deprecations Harvey Harrison
2008-02-01 10:54   ` Rafael J. Wysocki

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=07f44315b9cfad180296df99bcd38879f063f092.1201840183.git.len.brown@intel.com \
    --to=lenb@kernel.org \
    --cc=len.brown@intel.com \
    --cc=linux-acpi@vger.kernel.org \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-pm@lists.linux-foundation.org \
    --cc=rjw@sisk.pl \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®